require "Search_FX_GUI"

local my_dialog = nil

-------------------------------------------------------------------------------------------
--Main
-------------------------------------------------------------------------------------------
function search_track()

  -------------------------------------------------------------------------------------------
  --local functions
  -------------------------------------------------------------------------------------------
  
  --generic handler which accepts an optional parameter for dialog
  local function closer(d)
    
    if d then
     -- renoise.song().tracks_observable:remove_notifier(closer,my_dialog) --remove notifiers
  --  renoise.tool().app_release_document_observable:remove_notifier(closer, my_dialog)   
    end       
      
    if d and d.visible then             
      d:close()
    end
  end
  
  --function to focus the pattern editor
  local function focus_pattern_ed()
    renoise.app().window.lock_keyboard_focus = false
    renoise.app().window.lock_keyboard_focus = true
  end
  -------------------------------------------------------------------------------------------
  
  if (my_dialog and my_dialog.visible) then -- only allows one dialog instance
    my_dialog:show()
    return
  end
    
  local no_of_tracks = #renoise.song().tracks
  local matching_string = nil
  local current_track = renoise.song().selected_track_index 
  local current_track_name =  renoise.song().tracks[current_track].name 
  local vb = renoise.ViewBuilder()
  local field_typing = false
  
  --get available plugin infos--short names will be copied to names table, path used to luad plugin
  local vst_table = renoise.song().instruments[1].plugin_properties.available_plugin_infos
  
  --This is the list of strings that will appear in the pop-up
  local names_table = {}
  
  --populate table with the plugin short names
  for i = 1,#vst_table do
    names_table[i] = vst_table[i].short_name
  end
  
  --add a "No Match" field to the beginning
  table.insert(names_table,1,"No Match")
  --add a blank entry to vst table as we need the indexes to match
  table.insert(vst_table,1,{})
   
  ----------------------------------------------------------------------------
  --see what the longest name in the search list to set the GUI popup width by
  ----------------------------------------------------------------------------
  local function get_longest_name()
   --set name length to 1
   local name_length = 10
   --loop to set to longest string
   for i = 1, #names_table do --include ([1] "No Match)"
     if names_table[i]:len() > name_length then
       name_length = names_table[i]:len()
     end
   end
   return name_length
  end
   
  -------------------------------------------------------------------------------------------
  --GUI
  -------------------------------------------------------------------------------------------
  local dialog_content =
    
    vb:vertical_aligner {
    margin = 8,
    mode = "center",
                           
    vb:textfield {
    -- text = current_track_name,
    id = "textfield",
    --set width relative to the longest name in search table (at rough average of 7 pixels per char)
    width = (7 * get_longest_name()),
    },
                            
    vb:popup {
    id = "popup",
    --set width relative to the longest name in search table (at rough average of 7 pixels per char)
    width = (7 * get_longest_name()),
    value = 1,--"No Match"
    items = names_table,
    --updates the textfield when an entry is chosen from the pop-up menu
   -- notifier = function(number) vb.views["textfield"].value = names_table[number] end 
    },
    --spacer text
    vb:horizontal_aligner{
      vb:text{
       text = ""
      }
    },

    vb:horizontal_aligner{
      mode = "center",
      vb:button {
        width = 60,
        height = 20,
        text = "Load",
        notifier = function() --load vst from popup
          --if no value chosen then return
          if vb.views["popup"].value == 1 then  
            return
          end
          local inst_index = renoise.song().selected_instrument_index
          renoise.song().instruments[inst_index].plugin_properties:load_plugin(vst_table[vb.views["popup"].value].path)
          focus_pattern_ed()
           --open editor if preferenced
          if (options.auto_open.value) then -- auto_open then
            if renoise.song().instruments[inst_index].plugin_properties.plugin_device.external_editor_available then
              renoise.song().instruments[inst_index].plugin_properties.plugin_device.external_editor_visible = true
            end
          end   
          closer(my_dialog)
        end
      },
      vb:text{
       text = "Ed."
      },
      --option to auto-open the GUI on Load
      vb:checkbox {
        value =  options.auto_open.value, 
        notifier = function()  options.auto_open.value = not options.auto_open.value end,
        id = "gui check",
      }
    }
  }
  
  ----------------------------------------------------------------------------------------------------        
  -- key handler function()  
  ----------------------------------------------------------------------------------------------------
  local string_holder = ""
  local function my_keyhandler_func(dialog, key) 
   -- rprint(key)
      
   local full_string = vb.views["textfield"].value
   --as function is called before the textfield is updated we need to keep track
   --by adding the new character here
   if key.character then
     full_string = vb.views["textfield"].value..key.character
   end
   --remove end character if backspace
   if key.name == "back" then
     full_string = string.sub(vb.views["textfield"].value, 1, (#full_string - 1)) 
   end
   --remove all character if backspace + Ctrl
   if  (key.modifiers == "control" and key.name == "back") then
     full_string = "" 
   end
   
    -----------------------------------
    --function conataining loop that checks the current textfield string against 
    --the available targets in the table
    local function loop_to_match()
      --popup loop to see if match found
      for i = 2,#names_table do --search from 2 so we don`t include "No Match"
          
        local name_string = names_table[i]
        local name_string_holder = name_string                   
        local found_match = string.find(name_string,full_string)--normal

        if found_match ~= nil then --match found
          --renoise.song().selected_track_index = track
          vb.views["popup"].value = i
          break
        else  --no match so try lower case
          name_string = string.lower(name_string)
        
          found_match = string.find(name_string, full_string) --lower
          vb.views["popup"].value = i
          if found_match ~= nil then
            break
         else
            vb.views["popup"].value = 1                        
         end
       end
     end
   end
   ------------------------------------
    
   --exit when escape pressed
   if (key.modifiers == "" and key.name == "esc") then
     closer(my_dialog)
   end
    
    --toggle the GUI checkbox
    if key.name == "f1" or key.name == "f12" then
      vb.views["gui check"].value = (not vb.views["gui check"].value)
      return
    end
    
    --clear the textfield on ctrl + back
    if (key.modifiers == "control" and key.name == "back") then
      field_typing= true
      string_holder = ""
      vb.views["textfield"].value = string_holder
      vb.views["popup"].value = 1 --shows "No Match"
        
    --delete last character when back key pressed  
    elseif key.name == "back" then 
      field_typing= true
      local string_minus_1
      string_minus_1 = string.sub(string_holder, 1, (string.len(string_holder)-1))--remove last character from string
      vb.views["textfield"].value = string_minus_1
      string_holder = string_minus_1
      
      if full_string == "" then --all deleted
        vb.views["popup"].value = 1 --shows "No Match"
        return
      end
      --compare textfield to table
      loop_to_match()
    
    --------------------------------------------------------        
    --Execute (load the vst and exit) when return is pressed  
    -------------------------------------------------------- 
    elseif key.name == "return" then
      
      --when "No Match" do nothing and return
      if vb.views["popup"].value == 1 then
        return
      end
         
      --loop to find vst name that matches the textfield
      for i = 1,#names_table do 
        local name_string = names_table[i]
        local name_string_holder = name_string                  
        local found_match = string.find(name_string,full_string)--normal
       
        --if no match found then exit the tool
        if found_match ~= nil then
          local inst_index = renoise.song().selected_instrument_index
          renoise.song().instruments[inst_index].plugin_properties:load_plugin(vst_table[vb.views["popup"].value].path)   
          focus_pattern_ed()
          --open editor if preferenced
          if (options.auto_open.value) then -- auto_open then
            if renoise.song().instruments[inst_index].plugin_properties.plugin_device.external_editor_available then
              renoise.song().instruments[inst_index].plugin_properties.plugin_device.external_editor_visible = true
            end
          end  
          closer(my_dialog)
          break
        else
          name_string = string.lower(name_string) --try lower case
          --load vst here
          if vb.views["popup"].value ~= 1 then
            local inst_index = renoise.song().selected_instrument_index
            renoise.song().instruments[inst_index].plugin_properties:load_plugin(vst_table[vb.views["popup"].value].path)
            focus_pattern_ed()
            --open editor if preferenced
            if (options.auto_open.value) then -- auto_open then
              if renoise.song().instruments[inst_index].plugin_properties.plugin_device.external_editor_available then
                renoise.song().instruments[inst_index].plugin_properties.plugin_device.external_editor_visible = true
              end
            end        
            closer(my_dialog)
            break
          else
            vb.views["popup"].value = 1               
          end --if else
        end--for
      end 
      
    --Upwards Arrow chooses previous popup entry     
    elseif key.name == "down" then --character input
      if vb.views["popup"].value ~= 1 then
        vb.views["popup"].value = vb.views["popup"].value - 1
      end
    
    --down Arrow  chooses next popup entry      
    elseif key.name == "up" then --character input
      if vb.views["popup"].value ~= #names_table then
        vb.views["popup"].value = vb.views["popup"].value + 1 
      end 
      
    --any other character input      
    elseif key.character ~= nil then --character input
      field_typing = true
      string_holder = string_holder..key.character 
      --add character to textfield
      vb.views["textfield"].value = vb.views["textfield"].value..key.character 
      --compare textfield to table
      loop_to_match() 
    end  --elseif "ky chooser
  end                       
  ------------------------------------------------------------------------------------------
  ------------------------------------------------------------------------------------------
  --Script dialog
  my_dialog = renoise.app():show_custom_dialog(
      "Search VSTi", dialog_content,my_keyhandler_func)
  
  ------------------------------------------------------------------------------------------
  --notifiers
  ------------------------------------------------------------------------------------------
  renoise.tool().app_release_document_observable:add_notifier(closer, my_dialog)
  renoise.song().tracks_observable:add_notifier(closer,my_dialog)
  
end --end of main
-------------------------------------

--keybinding
renoise.tool():add_keybinding {
  name = "Global:Tools:`VFM` Search and Load VSTi",
  invoke = function()search_track()end
  
}
 
