--VSTiFromMenu.xrnx
require "Search GUI"
require "extra_shortcuts" 

------------------------------------------------------------------------
------------------------------------------------------------------------
--[1] Instrument Box Function --main()
------------------------------------------------------------------------

function main()

  --plugin infos from API
  local rns_plugin_infos = renoise.song().instruments[1].plugin_properties.available_plugin_infos
  local no_of_vsts = #rns_plugin_infos
  
  ---------------------------------------------------
  --load vsti
  ---------------------------------------------------
  
  local function load_vsti(fn_no)
  
    local inst_index = renoise.song().selected_instrument_index
    renoise.song().instruments[inst_index].plugin_properties:load_plugin(rns_plugin_infos[fn_no].path)
    -- if auto_open then open GUI
    if (options.auto_open.value) then
      if renoise.song().instruments[inst_index].plugin_properties.plugin_device.external_editor_available then
        renoise.song().instruments[inst_index].plugin_properties.plugin_device.external_editor_visible = true
      end
    end
  end --local function
  
  
  local plug_inst_type = {}
  
  for VSTs = 1,no_of_vsts do
    --Plugins
    if  string.find(rns_plugin_infos[VSTs].path , "VST") then
      plug_inst_type[VSTs] = "VST" 
      
    elseif  string.find(rns_plugin_infos[VSTs].path , "DSSI") then
      plug_inst_type[VSTs] = "DSSI"
  
    elseif  string.find(rns_plugin_infos[VSTs].path ,"AU") then
      plug_inst_type[VSTs] = "AU"
    end
    
  end
  
  ---------------------------------------------------
  --Add VST menu entries with loop
  ---------------------------------------------------
  
  if no_of_vsts then --needed for when no vsts available  
    local menu_entry = ""
    for i = 1,no_of_vsts do
      if plug_inst_type[i] == "VST" then 
        --is_bridged? 
        if rns_plugin_infos[i].is_bridged then
          menu_entry = "Instrument Box:Load VSTi..: * "..rns_plugin_infos[i].short_name
        else
          menu_entry = "Instrument Box:Load VSTi..:"..rns_plugin_infos[i].short_name
        end
         --add to menu 
        if not renoise.tool():has_menu_entry(menu_entry) then
            renoise.tool():add_menu_entry {
              name = menu_entry,
              invoke = function() load_vsti(i) end
            }
        end
      end
    end
  end
  
  ---------------------------------------------------
  --Add menu entries with loop
  ---------------------------------------------------
  
  if no_of_vsts then --needed for when no vsts available  
    local menu_entry = ""
    for i = 1,no_of_vsts do
      if plug_inst_type[i] == "DSSI" then 
        --is_bridged? 
        if rns_plugin_infos[i].is_bridged then
          menu_entry = "Instrument Box:Load DSSI..: * "..rns_plugin_infos[i].short_name
        else
          menu_entry = "Instrument Box:Load DSSI..:"..rns_plugin_infos[i].short_name
        end
         --add to menu 
        if not renoise.tool():has_menu_entry(menu_entry) then
            renoise.tool():add_menu_entry {
              name = menu_entry,
              invoke = function() load_vsti(i) end
            }
        end
      end
    end
  end
  
  ---------------------------------------------------
  --Add AU menu entries with loop
  ---------------------------------------------------
  
  if no_of_vsts then --needed for when no vsts available  
    local menu_entry = ""
    for i = 1,no_of_vsts do
      if plug_inst_type[i] == "AU" then 
        --is_bridged? 
        if rns_plugin_infos[i].is_bridged then
          menu_entry = "Instrument Box:Load AU..: * "..rns_plugin_infos[i].short_name
        else
          menu_entry = "Instrument Box:Load AU..:"..rns_plugin_infos[i].short_name
        end
         --add to menu 
        if not renoise.tool():has_menu_entry(menu_entry) then
            renoise.tool():add_menu_entry {
              name = menu_entry,
              invoke = function() load_vsti(i) end
            }
        end
      end
    end
  end
  
  ----------------------------------------------------
  --Now add Preference menu entry to auto open VST GUI
  ----------------------------------------------------
  
  -- triple dash: "---" creates menu divider 
  renoise.tool():add_menu_entry {
       name = "--- Instrument Box:Load VSTi..: Auto Open Gui", 
       selected = function() return options.auto_open.value end, 
       invoke = function() options.auto_open.value = not options.auto_open.value end
       }
  
  
  ----------------------------------------------------
  --remove notifier so only runs on renoise start-up
  ----------------------------------------------------
  
  if renoise.tool().app_new_document_observable:has_notifier(main) then
   renoise.tool().app_new_document_observable:remove_notifier(main)
  end     
end  --main

--------------------------------------------------------------------------
--------------------------------------------------------------------------


-------------------------------------------------------------------------
-- [2] MIXER function
-------------------------------------------------------------------------
function add_vsts_to_mixer()

local rns_track_device_infos = renoise.song().tracks[1].available_device_infos

local plug_type = {}
local VSTs = 1
local no_of_vsts = #rns_track_device_infos

local hidden_plugs = {"*Formula Device",
                "*MIDI-CC Device",
              --"Distortion",
              --"Filter",
                "Filter 2",
              --"Gate",
              --"LofiMat",
                "MasterTrackVolPan",
              --"mpReverb",
                "SendTrackVolPan",
                "Shaper",
                "Stutter",
                "TrackVolPan"
                }
                                                
                
--get fx plugin names and add to table "plugins"
for VSTs = 1,no_of_vsts do
  
  --routing
  if string.find(rns_track_device_infos[VSTs].path , "#")then
    plug_type[VSTs] = "Routing"
  --meta
  elseif string.find(rns_track_device_infos[VSTs].path , "*") then
    plug_type[VSTs] = "Meta" 
  --native is last as all renoise fx have native in path
  elseif string.find(rns_track_device_infos[VSTs].path , "Native")then
    plug_type[VSTs] = "Native"
  end
 
  --Plugins
  if  string.find(rns_track_device_infos[VSTs].path , "VST") then
    plug_type[VSTs] = "VST" 
    
  elseif  string.find(rns_track_device_infos[VSTs].path , "DSSI") then
    plug_type[VSTs] = "DSSI"

  elseif  string.find(rns_track_device_infos[VSTs].path ,"AU") then
    plug_type[VSTs] = "AU"
  end
  
  -- remove hidden/ compatability plugs
  if plug_type[VSTs] == "Native" then
    for i = 1,13 do
      if rns_track_device_infos[VSTs].short_name == hidden_plugs[i] then
        plug_type[VSTs] = "Void"
        break
      end
    end
  end
end--for

---------------------------------------------------
--load VST fx on execution
---------------------------------------------------

local function load_vst(fn_no)
 --get total devices in current tracks chain
  local track = renoise.song().selected_track_index
  local track_device_total = #renoise.song().selected_track.devices
  local current_track = renoise.song().tracks[track]
  
  --load
  current_track:insert_device_at(rns_track_device_infos[fn_no].path, (track_device_total + 1))
  --select device
  renoise.song().selected_track_device_index = (track_device_total + 1)
  
  --open editor if preferenced
  if (options.auto_open_fx.value) then -- auto_open then
    if current_track.devices[(track_device_total + 1)].external_editor_available then
      current_track.devices[(track_device_total + 1)].external_editor_visible = true
    end
  end 
end


------------------------------------
--Add Menu entries with loops
------------------------------------

--table for each menu area in renoise to be added
local menu_roots = {"Mixer:","DSPs Chain:","DSP Device:"} --"Track DSPs Chain" is now "DSPs Chain"
--initialise menu string, it will be concatenated to root
local menu_entry = "" 


---------------------------------------------------
--Add VST entries with loops
---------------------------------------------------
for i = 1,no_of_vsts do

  if plug_type[i] == "VST" then 
   --add * if it is a bridged plugin
    if rns_track_device_infos[i].is_bridged then
      menu_entry = "Load VST fx..: * "..rns_track_device_infos[i].short_name
    else
      menu_entry = "Load VST fx..:"..rns_track_device_infos[i].short_name
    end
    --add menu entry
    for root = 1,#menu_roots do
      if not renoise.tool():has_menu_entry(menu_roots[root]..menu_entry) then
        -- add entries
        renoise.tool():add_menu_entry {
         name = menu_roots[root]..menu_entry,
         invoke = function() load_vst(i) end
        }
      end
    end
  end
end


---------------------------------------------------
--Add DSSI entries with loop
---------------------------------------------------
 
for i = 1,no_of_vsts do
  
  if plug_type[i] == "DSSI" then
  
      --add * if it is a bridged plugin
    if rns_track_device_infos[i].is_bridged then
      menu_entry = "Load DSSI fx..: * "..rns_track_device_infos[i].short_name
    else
      menu_entry = "Load DSSI fx..:"..rns_track_device_infos[i].short_name
    end
    --add all menu entry
    for root = 1,#menu_roots do
      if not renoise.tool():has_menu_entry(menu_roots[root]..menu_entry) then
        -- add entries
        renoise.tool():add_menu_entry {
         name = menu_roots[root]..menu_entry,
         invoke = function() load_vst(i) end
        }
      end
    end
  end
end


---------------------------------------------------
--Add AU entries with loop
---------------------------------------------------

for i = 1,no_of_vsts do
 
  if plug_type[i] == "AU" then
    
   --add * if it is a bridged plugin
    if rns_track_device_infos[i].is_bridged then
      menu_entry = "Load AU fx..: * "..rns_track_device_infos[i].short_name
    else
      menu_entry = "Load AU fx..:"..rns_track_device_infos[i].short_name
    end
    
    --add menu entry
    for root = 1,#menu_roots do
      if not renoise.tool():has_menu_entry(menu_roots[root]..menu_entry) then
        -- add entries
        renoise.tool():add_menu_entry {
         name = menu_roots[root]..menu_entry,
         invoke = function() load_vst(i) end
        }
      end
    end
  end
end


---------------------------------------------------
--Add Native entries with loops
---------------------------------------------------

local add_divider = "---"

for i = 1,no_of_vsts do

  if plug_type[i] == "Native"  then
    menu_entry = "Load Native fx..:"..rns_track_device_infos[i].short_name
    
    --add menu entries
    for root = 1,#menu_roots do
      -- add entries
      renoise.tool():add_menu_entry {
        name = add_divider..menu_roots[root]..menu_entry,
        invoke = function() load_vst(i) end
        }
     end

     add_divider = "" --only add a single divider at the beginning
  end
end

---------------------------------------------------
--Add Meta Devices with loops
---------------------------------------------------

for i = 1,no_of_vsts do
    
  if plug_type[i] == "Meta"  then
    menu_entry = "Load Native fx..:Meta fx..:"..rns_track_device_infos[i].short_name
 
 --add menu entries
    for root = 1,#menu_roots do
      -- add entries
      renoise.tool():add_menu_entry {
        name = menu_roots[root]..menu_entry,
        invoke = function() load_vst(i) end
        }
    end
  end
end

---------------------------------------------------
--Add Routing Devices with loop
---------------------------------------------------

for i = 1,no_of_vsts do
    
  if plug_type[i] == "Routing"  then
    menu_entry = "Load Native fx..:Routing fx..:"..rns_track_device_infos[i].short_name

   --add menu entries
    for root = 1,#menu_roots do
      -- add entries
      renoise.tool():add_menu_entry {
        name = menu_roots[root]..menu_entry,
        invoke = function() load_vst(i) end
        }
    end
  
  end
end

-------------------------------------------------------------------
--Now add Preference menu as last entry entry to auto open VST GUI
-------------------------------------------------------------------
-- triple dash: "---" creates menu divider 

   --add menu entries
    for root = 1,#menu_roots do
      renoise.tool():add_menu_entry {
           name = "--- "..menu_roots[root].."Load VST fx..: Auto Open Gui", 
           selected = function() return options.auto_open_fx.value end, 
           invoke = function() options.auto_open_fx.value = not options.auto_open_fx.value end
           }
    end

----------------------------------------------------
--remove notifier so only runs on renoise start-up
----------------------------------------------------
if renoise.tool().app_new_document_observable:has_notifier(add_vsts_to_mixer) then
 renoise.tool().app_new_document_observable:remove_notifier(add_vsts_to_mixer)
end

end --main



--------------------------------------------------
--Global add new document notifier
--------------------------------------------------
renoise.tool().app_new_document_observable:add_notifier(main)
renoise.tool().app_new_document_observable:add_notifier(add_vsts_to_mixer)



--]]


-----------------------------------------------------------------------------
--[[local function show_automated_devices_gui()

-- Needed + requested on forum:
--renoise.song().patterns[].tracks[].selected_automation_device_index, _observable

local  i = 1
local track_1 = renoise.song().patterns[1].tracks[1]

  while track_1.automation[i] do

   local track_device = track_1.automation[i].dest_device
 
   if track_device.external_editor_available then
     track_device.external_editor_visible = true
   end

    i = i + 1
  end
end

for 
renoise.tool():add_menu_entry {
     name = "Track Automation:Show Device Editor:",
     invoke = function ()  show_automated_devices_gui() end
     }
     
end--]]






 
